const DOM = (() => {

  /**
   * @private
   * New instance of @class Log
   * @constant Object
   */ 
  const log = new Log();

  /**
   * @private
   * New instance of @class Spinal
   * @constant Object
   */ 
  const spnl = new Spinal();

  /**
   * @private
   * New instance of @class DB
   * @constant Object
   */ 
  const subDb = new DB();

  /** 
   * @private
   * Operations for info fields
   * @constant object 
   */  
  const operations = {
    sub : (num1, num2) => num1 - num2,
    add : (num1, num2) => num1 + num2
  }

  /** 
   * @private
   * Instance of the DOM class
   * @var Object 
   */
  var that;
  
  let searching = false;

  /** 
   * @public
   * @constructor
   * @param mainId  Id of the main html tag  (optinal)
   */  
  function DOM (mainId = null)
  {
    that = this;

    /**
     * @public
     * The main content holder tag
     * @var DOM
     */    
    this.mainId = document.getElementById(mainId);

    this.parent;
    /** 
     * @global
     * The standard login page
     * @var void 
     */    
    window.loginPage    = this.login;

    /** 
     * @global
     * @the standard settings page
     * @var void 
     */    
    window.settingsPage = this.settings;

    this.addBalance = false;
  }

  /** 
   * @public
   * Load HTML Template from another files
   * @param path      The path of the HTML template file
   * @param callback  The callback to pass response result
   * @returns Callback void or boolean
   */  
  DOM.prototype.loadDOM = (path, callback = null) => {
    if (log.checkType({"s:path":path}) === false) return;

    try
    {
      // The XHR connection to load HTML templates
      var xhr = new XMLHttpRequest();
      xhr.onload = () => {
        if (callback !== null)
        {
          if (log.checkType({"f:callback":callback}) === false) return;
          else callback(xhr.responseText);
        }
        else return xhr.responseText;
      }
      xhr.open("GET", path);
      xhr.responseType = "text";
      xhr.send();
    }
    catch (error)
    {
      console.error(error);
    }
  }

  /** 
   * @public
   * Create login form DOM template and control its functionality
   * @param callback Callback for logging in 
   */  
  DOM.prototype.login = (callback = null) => {
    that.loadDOM("././templates/login.html", html => {
      that.mainId.innerHTML += html;

      let user     = document.getElementById("username");
      let password = document.getElementById("password");
      let submit   = document.getElementById("login-submit");
      let checkbox = document.querySelector("#remember .cbx");
      let message  = document.getElementById("alert");
      let login    = document.getElementById("login");
      let loginSet = document.getElementById("login-settings");

      if (localLang === "ar") login.classList.add("right");

      loginSet.addEventListener("click", () => settingsPage(true));

      if (callback !== null)
      {
        if (log.checkType({"f:callback":callback}) === false) return;

        // "Remember me" checkbox
        checkbox.addEventListener("click", () => {
          checkbox.classList.toggle("checked");
        });

        // The submition to check and control user login information
        // Here it sends a log in request to check information
        submit.addEventListener("click", () => {
          callback(user.value, password.value, data => {
            
            console.log(data);
            if (data.error_code == 4)
            {
              // Show error message if the log in information are not correct
              message.classList.add("show");
              setTimeout(() => {
                message.classList.remove("show");
              }, 2000);
            }
            else if (typeof data.Session !== undefined)
            {
              // Store user information and open session if the log in information are correct
              if (checkbox.classList.contains("checked"))
              {
                localStorage.setItem("user", user.value);
                localStorage.setItem("password", password.value);
              }
              else
              {
                sessionStorage.setItem("user", user.value);
                sessionStorage.setItem("password", password.value);
              }
              
              localStorage.setItem("company", data.CompanyName);
              localStorage.setItem("dataName", data.DataName);
              window.location.reload();
            }
          });
        });
      }
      spinal.translate();
      spnl.execute();
    });
  }
  
  /** 
   * @public
   * Create body DOM template and control its functionality
   */
  DOM.prototype.body = () => {
    try
    {
      that.loadDOM("././templates/body.html", html => {
        that.mainId.innerHTML += html;
        spnl.execute();
      });
    }
    catch (error)
    {
      console.error(error);
    }
  }
  
  /** 
   * @public
   * Create home DOM template and control its functionality
   */
  DOM.prototype.home = () => {
    try
    {
      that.hideBack();
      document.removeEventListener("backbutton", dom.backEvents);
      DB.prototype.store("s:r", "parentPage");
      DB.prototype.store("s:r", "searching");
      DB.prototype.store("s:r", "NameSelected");
      // Empty DOM holder tag
      parent = document.getElementById("content");
      while (parent.firstChild) {
        parent.removeChild(parent.firstChild);
      }

      that.loadDOM("././templates/home.html", html => {
        let wrapper = document.getElementById("content");
        wrapper.innerHTML += html;

        // Load home functions
        homeFunctions();
        spnl.execute();
      });
    }
    catch (error)
    {
      console.error(error);
    }
  }

  /** 
   * @public
   * Create settings DOM template and control its functionality
   * @param show  Show or hide the settings at the first loading
   */
  DOM.prototype.settings = (show = false) => {
    try
    {
      that.loadDOM("././templates/settings.html", html => {
        that.mainId.innerHTML += html;

        // Show the settings page if the @var "show" is set as true
        if (show == true)
        {
          document.getElementById("settings").classList.add("show-settings");
          SettingsFunctions(true);
          spinal.translate();
        }
        else spnl.execute();
      });
    }
    catch (error)
    {
      console.error(error);
    }
  }

  /** 
   * @public
   * Create table DOM template and control its functionality
   */
  DOM.prototype.table = {

    /** 
     * @public
     * Create the the table DOM and prepare it to hold data 
     * 
     * @param parent    The parent tag of Created table
     * @param id        The id attribute of the table
     * @param classess  The classes of the table (optional params)
     */    
    create: (parent, id, addBalance = false, ...classes) => {
      if (log.checkType({"s:parent":parent, "s:id":id}) === false) return;
        for (const cls of classes)
          if (log.checkType({"s:cls":cls}) === false) return;
        
      try
      {
        parent = document.querySelector(parent);

        // Empty DOM holder tag
        while (parent.firstChild) {
          parent.removeChild(parent.firstChild);
        }

        // Remove scroll height
        parent.scrollTop = 0;

        DB.prototype.store("s:s", "currentPage", id);

        let mainTitle;
        if (searching === false || DB.prototype.store("s:g", "searching") === "true")
        {
          mainTitle = document.createElement("h1");
          mainTitle.id = "main-title";
          let parentPage  = DB.prototype.store("s:g", "parentPage");
          let currentPage = DB.prototype.store("s:g", "currentPage");

          if (parentPage === currentPage)
            mainTitle.innerHTML = spinal.language[DB.prototype.store("l:g", "localLanguage")][parentPage];
          else mainTitle.innerHTML = DB.prototype.store("s:g", "NameSelected");

          parent.appendChild(mainTitle);
        }

        // DOM template
        that.table.body = document.createElement("div");
        that.table.body.id = id;
        that.table.body.classList.add("content-body", ...classes);
        if (typeof mainTitle !== "undefined")
          that.table.body.style.height = "calc(100% - 80px)";
        parent.appendChild(that.table.body);

        that.addBalance = addBalance;
      }
      catch (error)
      {
        console.error(error);
      }

      if (DB.prototype.store("s:g", "currentPage") === DB.prototype.store("s:g", "parentPage"))
      {
        that.parent = parent.childNodes[1];
      }
    },

    /** 
     * @public
     * Fill the table with the given data
     * 
     * @param data            The data to display
     * @param hiddenContent   The custom DOM of the hidden actions area
     * @param codeField       The data key name for the code field in every single row of table
     * @param titleField      The data key name for the title field in every single row of table 
     * @param infoFields      The data key names for the info fields in every single row of table
     */    
    fill : (data, hiddenContent, codeField, titleField , reverse, ...infoFields) => {
      if (hiddenContent !== false)
        if (log.checkType({"a:hiddenContent":hiddenContent}) === false) return;

      if (typeof data === "undefined") return;
      // Return if it is no data to show
      if (typeof data.error_code !== "undefined" && data.error_code == 1002)
      {
        let row = document.createElement("div");
        row.classList.add("table-row", "no-data");
        row.innerHTML = "<h3>" + spinal.language[localLang].noResult + "<h3>";
        that.table.body.appendChild(row);
        return;
      }

      that.color = colors[DB.prototype.store("s:g", "currentPage")];

      let shadow;
      if (document.getElementById("shadow") == null)
      {
        shadow = document.createElement("div");
        shadow.id = "shadow";
        if (DB.prototype.store("s:g", "currentPage") === DB.prototype.store("s:g", "parentPage"))
          that.parent.appendChild(shadow);
        else that.table.body.appendChild(shadow);
      }
      else shadow = document.getElementById("shadow");

      let balance = 0;

      if (reverse === true) {
        data = data.reverse();
        let num = DB.prototype.store("s:g", "ElmSelected");
        balance = db.data.accounts[num].ActDebit - db.data.accounts[num].ActCredit;
      }
      for (let i = 0; i < data.length; i++)
      {
        if (typeof data.error_code !== "undefined") return;

        // Create element tag
        let element = document.createElement("div");
        element.classList.add("element");
        if (DB.prototype.store("s:g", "currentPage") === DB.prototype.store("s:g", "parentPage"))
          that.parent.appendChild(element);
        else that.table.body.appendChild(element);

        // Create element row tag
        let row = document.createElement("div");
        row.classList.add("table-row");
        if (hiddenContent === false) row.classList.add("no-action");
        if (titleField === null) row.classList.add("no-title");
        element.appendChild(row);

        // Create element row head tag
        let rowHead = document.createElement("div");
        rowHead.classList.add("row-head");
        row.appendChild(rowHead);

        // Create the special code field in row head tag
        let code = document.createElement("div");
        codeVal = typeof codeField === "object" ? codeField.name : codeField;
        if (data[i][codeVal].toString().includes('.') && !isNaN(data[i][codeVal]))
          code.innerHTML = data[i][codeVal].toFixed(2);
        else if (/^\d{4}-\d{2}-\d{2}$/.test(data[i][codeVal]))
          code.innerHTML = data[i][codeVal].substring(5, 11);
        else code.innerHTML = data[i][codeVal];
        code.classList.add("code");
        code.style.background = that.color;
        rowHead.appendChild(code);

        // Create the special title field in row head tag
        let title = document.createElement("div");
        titleField = (typeof titleField === "object" && titleField !== null) ? titleField.name : titleField;
        title.innerHTML = data[i][titleField];
        title.classList.add("main-title");
        rowHead.appendChild(title);

        let rowBody;

        if (hiddenContent.length > 0) {
          // Create the hidden actions area
          let hidden = document.createElement("div");
          if (typeof data[i].ElmID !== "undefined") hidden.id = data[i].ElmID;
          hidden.classList.add("hidden-area");
          row.appendChild(hidden);

          // Fill the hidden area with the information of @param "hiddenContent"
          for (let g = 0; g < hiddenContent.length; g++)
          {
            let options = document.createElement("div");
            options.classList.add("action-option");
            if (hiddenContent.length > 3)
              options.style.width = "calc(100% / 3)";
            else options.style.width = "calc(100% / " + hiddenContent.length + ")";
            options.addEventListener("click", () => {
              DB.prototype.store("s:s", "ElmSelected", data[i].ElmID);
              DB.prototype.store("s:s", "NameSelected", data[i].ElmName);
              if (DB.prototype.store("s:g", "currentPage") === DB.prototype.store("s:g", "parentPage"))
              {
                parent.classList.add("fade-out");
                setTimeout(() => {
                  parent.classList.remove("fade-in");
                  parent.classList.remove("fade-out");
                }, 900);
                setTimeout(() => {
                  document.getElementById("child-content").classList.add("show");
                }, 300);
              }
              hiddenContent[g][2]();
            });
            hidden.appendChild(options);

            let icon = document.createElement("div");
            icon.classList.add("option-icon");
            icon.innerHTML = hiddenContent[g][0];
            options.appendChild(icon);

            let name = document.createElement("div");
            name.innerHTML = spinal.language[localLang][hiddenContent[g][1]];
            options.appendChild(name);
          }

          // Create the row body tag
          rowBody = document.createElement("div");
          rowBody.classList.add("row-body");
          row.appendChild(rowBody);

          // Create the actions displayer button
          let actions = document.createElement("div");
          actions.innerHTML = "<i class=\"fas fa-ellipsis-v\"></i>";
          actions.classList.add("actions");
          rowBody.appendChild(actions);
          actions.addEventListener("click", () => {
            let activeRow = document.querySelector(".table-row.active");
            if (activeRow !== null && !row.classList.contains("active"))
              activeRow.classList.remove("active");
            row.classList.toggle("active");
          });
        }

        if (infoFields.length > 0) {
          if (typeof rowBody === "undefined") {
            // Create the row body tag
            rowBody = document.createElement("div");
            rowBody.classList.add("row-body");
            row.appendChild(rowBody);
          }
          // Create the more info side of row body
          let moreInfo = document.createElement("div");
          moreInfo.classList.add("more-info");
          rowBody.appendChild(moreInfo);

          // Fill the more information side with the information of @params "infoFields"
          for (let j = 0; j < infoFields.length; j++)
          {
            // Make sure that there no field in more info side more than three
            if (j === 3)  break;

            // Create the info field for every single infoFields value
            let itemInfo = document.createElement("div");
            itemInfo.classList.add("item-info");
            if (infoFields.length <= 3) itemInfo.style.width = "calc(93% / " + infoFields.length + ")";
            else itemInfo.style.width = "calc(93% / 3)";
            moreInfo.appendChild(itemInfo);

            // Create the name tag for the info kind field
            let nameItem = document.createElement("div");

            if (infoFields[j] !== "string")
            {
              nameItem.innerHTML = infoFields[j].name;
              if (typeof infoFields[j].color === "string")
                nameItem.style.color = infoFields[j].color;
            }
            else
            {
              nameItem.innerHTML = infoFields[j];
            }
            nameItem.classList.add("name-item");
            itemInfo.appendChild(nameItem);

            // Create the value tag for the info kind field
            let valueItem = document.createElement("div");
            
            if (typeof infoFields[j] === "string")
            {
              // If the value is a number, and the number is a decimal
              // display only 2 digits after decimal point
              if (isNaN(data[i][infoFields[j]]) === false)
              {
                valueItem.innerHTML = data[i][infoFields[j]].toString().includes('.') ? data[i][infoFields[j]].toFixed(2) : data[i][infoFields[j]];
              }
              
              else valueItem.innerHTML = data[i][infoFields[j]];
            }
            else if (typeof infoFields[j] === "object")
            {
              if (operations.hasOwnProperty(Object.keys(infoFields[j])[1]))
              {
                // If the field key is an object then it must be a mathematical operation
                let result = operations[Object.keys(infoFields[j])[1]](
                  data[i][infoFields[j][Object.keys(infoFields[j])[1]][0]],
                  data[i][infoFields[j][Object.keys(infoFields[j])[1]][1]]
                );
                if (that.addBalance === true)
                {
                  if (reverse === true) {
                    balance = i == 0 ? balance : balance - result;
                  } else balance += result;
                  valueItem.innerHTML = balance;
                }
                else valueItem.innerHTML = result;
              }
              else if (typeof infoFields[j].key === "string")
              {
                valueItem.innerHTML = data[i][infoFields[j].key];
              }

              if (typeof infoFields[j].color === "string")
                valueItem.style.color = infoFields[j].color;
            }
            if (valueItem.innerHTML.toString().includes('.') && !isNaN(valueItem.innerHTML))
              valueItem.innerHTML = Number(valueItem.innerHTML).toFixed(2);

            valueItem.classList.add("value-item");
            itemInfo.appendChild(valueItem);
          }

          let currentPage = DB.prototype.store("s:g", "currentPage");
          if (currentPage === "matBillSearch" || currentPage === "actBillSearch" || currentPage === "actLastMoveSearch") {
            element.addEventListener("click", () => {
              DB.prototype.store("s:s", "ElmSelected", data[i].ElmID);
              DB.prototype.store("s:s", "NameSelected", data[i].ElmName);
              document.getElementById("child-content").classList.add("show");
              if (DB.prototype.store("s:g", "currentPage").includes("Last")) {
                Tables.actLastMoves();
              } else if (DB.prototype.store("s:g", "currentPage").includes("mat")) {
                Tables.matBills();
              } else {
                Tables.actBills();
              }
            });
          }
        } else {
          element.addEventListener("click", () => {
            DB.prototype.store("s:s", "ElmSelected", data[i].ElmID);
            DB.prototype.store("s:s", "NameSelected", data[i].ElmName);
            document.getElementById("child-content").classList.add("show");
            if (DB.prototype.store("s:g", "currentPage").includes("Last")) {
              Tables.actLastMoves();
            } else if (DB.prototype.store("s:g", "currentPage").includes("mat")) {
              Tables.matBills();
            } else {
              Tables.actBills();
            }
          });
        }

        if (titleField !== null)
        {
          // Create the popup for every single element row
          let popup = document.createElement("div");
          popup.classList.add("popup-card");
          element.appendChild(popup);
          
          // create the table for the information popup
          let popTable = document.createElement("table");
          popup.appendChild(popTable);
          
          // Create the special code field in popup tag
          let popCode = document.createElement("tr");
          popCode.style.background = that.color;
          let popCodeVal = "";
          if (data[i][codeVal].toString().includes('.') && !isNaN(data[i][codeVal]))
            popCodeVal = data[i][codeVal].toFixed(2);
          else popCodeVal = data[i][codeVal];
          popCode.innerHTML  = "<td>" + popCodeVal + "</td>";
          popCode.innerHTML += "<td>" + (typeof codeField === "object" ? spinal.language[localLang][codeField.key] : codeField) + "</td>";
          popTable.appendChild(popCode);

          // Create the special title field in popup tag
          let popTitle = document.createElement("tr");
          popTitle.innerHTML = "<td colspan=\"2\">" + data[i][titleField] + "</td>";
          popTable.appendChild(popTitle);

          // Display all iformation of @params "infoFields" in popup tag
          for (let h = 0; h < infoFields.length; h++)
          {
            let popInfo = document.createElement("tr");
            let infoResult;

            if (typeof infoFields[h] === "string") infoResult = data[i][infoFields[h]];
            else if (typeof infoFields[h] === "object")
            {
              if (operations.hasOwnProperty(Object.keys(infoFields[h])[1]))
              {
                if (balance.toString().includes('.') && !isNaN(balance))
                  infoResult = Number(balance).toFixed(2);
                else infoResult = balance;
              }
              
              if (typeof infoFields[h].key === "string")
              {
                infoResult = data[i][infoFields[h].key];
              }
            }

            if (typeof infoFields[h].display === "undefined" || infoFields[h].display !== false) {
              if (typeof infoFields[h].hide === "undefined" || infoFields[h].hide !== infoResult) {
                popInfo.innerHTML  = "<td colspan=\"1\">" + infoResult + "</td>";
                popInfo.innerHTML += "<td colspan=\"1\">" + (typeof infoFields[h] === "object" ? infoFields[h].name : infoFields[h]) + "</td>";
                popTable.appendChild(popInfo);
              }
            }
          }

          // Set the timer to control the viewing of popup
          // window as the user is starting, leaving and ending touch
          let timer;
          row.addEventListener("touchstart", () => {

            timer = setTimeout(() => {
              shadow.style.background = "rgba(0, 0, 0, 0.3)";
              popup.classList.add("active");
            }, 800);
          });

          row.addEventListener("touchend", () => {
            clearTimeout(timer);
            shadow.style.background = "rgba(0, 0, 0, 0)";
            popup.classList.add("hide");
            setTimeout(() => {
              popup.classList.remove("active");
              popup.classList.remove("hide");
            }, 290);
          });

          document.getElementById("content").addEventListener("scroll", () => {
            clearTimeout(timer);
          });
        }
      }
    },
    
    /** 
     * @public
     * Fill the table DOM with data.
     * Display and load more data by offset of @var "offset"
     * and with limit of DB query standard 
     * Fill the table with the given data
     * 
     * @param data            The data to display
     * @param hiddenContent   The custom DOM of the hidden actions area
     * @param codeField       The data key name for the code field in every single row of table
     * @param titleField      The data key name for the title field in every single row of table 
     * @param infoFields      The data key names for the info fields in every single row of table
     * 
     * @returns void "that.fill(...params)"
     */    
    scrollLoader: (data, hiddenContent, codeField, titleField, reverse, ...infoFields) => {
      let offset = 0;
      offset = limitFill(offset, data, hiddenContent, codeField, titleField, reverse, ...infoFields);
      if (that.table.body.id.includes("Search") && DB.prototype.store("s:g", "currentPage") !== DB.prototype.store("s:g", "parentPage"))
        that.table.body.addEventListener("scroll", (e) => {
          if (e.srcElement.scrollTop + e.srcElement.clientHeight >= (e.srcElement.scrollHeight / 100) * 90) {
            offset = limitFill(offset, data, hiddenContent, codeField, titleField, reverse, ...infoFields);
          }
        });
      else that.parent.addEventListener("scroll", (e) => {
        if (e.srcElement.scrollTop + e.srcElement.clientHeight >= (e.srcElement.scrollHeight / 100) * 90) {
          offset = limitFill(offset, data, hiddenContent, codeField, titleField, reverse, ...infoFields);
        }
      });
    }
  }

  function limitFill (offset, data, hiddenContent, codeField, titleField, reverse, ...infoFields) {
    let count = 0;
    let newData = [];
    for (let i = offset; i < data.length; i++) {
      if (data[i] !== -1)
      {
        if (count == 20) break;
        newData.push(data[i]);
        offset = i;
        count++
      }
    }
    that.table.fill(newData, hiddenContent, codeField, titleField, reverse, ...infoFields);
    return offset + 1;
  }

  /** 
   * @public
   * Show back button
   */
  DOM.prototype.showBack = () => {
    let back = document.getElementById("back-button");
    let search = document.getElementById("search");
    back.classList.remove("remove");
    search.classList.remove("remove");
  }

  /** 
   * @public
   * Hide back button
   */
  DOM.prototype.hideBack = () => {
    let back = document.getElementById("back-button");
    let search = document.getElementById("search");
    back.classList.add("remove");
    search.classList.add("remove");
  }

  /** 
   * @public
   * The functions of DOM templates
   * @returns void 
   */  
  DOM.prototype.functions = () => {
    //============BODY DOM FUNCTIONS============//
    bodyFunctions();

    //============SETTINGS DOM FUNCTIONS============//
    SettingsFunctions();

    spnl.execute();
  }

  /** 
   * @private
   * Body DOM functions 
   */  
  function bodyFunctions ()
  {
    let content     = document.getElementById("content");
    let menuBtn     = document.getElementById("menu-button");
    let searchBtn   = document.getElementById("search-icon");
    let searchBar   = document.getElementById("search-bar");
    let searchTbx   = document.getElementById("search-tbx");
    let backBtn     = document.getElementById("back-button");
    let wrapper     = document.getElementById("wrapper");
    let hiddenMenu  = document.getElementById("hidden-menu");
    let wrprClass   = document.querySelector(".wrapper");
    let header      = document.getElementById("header");
    let settings    = document.getElementById("settings");
    let contentItem = document.querySelectorAll(".content-item");
    // Hidden menu buttons
    let homeOpt     = document.getElementById("homeOpt");
    let materials   = document.getElementById("materials");
    let accounts    = document.getElementById("accounts");
    let settingsBtn = document.getElementById("settings-btn");
    let logoutBtn   = document.getElementById("logout-btn");
    
    if (localLang == "ar")
    {
      wrapper.classList.add("right");
      hiddenMenu.classList.add("right");
    }

    // Show hidden menu
    menuBtn.addEventListener("click", () => {
      wrprClass.classList.toggle("show-nav");
    });

    // Hide hidden menu
    let sideBar = document.getElementById("hidden-menu");
    sideBar.addEventListener("click", () => {
      wrapper.classList.remove("show-nav");
    });

    // Show search bar
    searchBtn.addEventListener("click", () => {
      backBtn.classList.add("hide");
      menuBtn.classList.add("hide");
      searchBtn.classList.add("hide");
      setTimeout(() => {
        searchBar.classList.add("show-bar");
        document.getElementById("search-tbx").focus();
      }, 200);
    });

    let childContent = document.getElementById("child-content");
    let searchTimer;
    searchTbx.addEventListener("input", () => {
      if (typeof searchTimer !== "undefined");
        clearTimeout(searchTimer);

      let currentPage = DB.prototype.store("s:g", "currentPage");
      searchTimer = setTimeout(() => {
        if (searchTbx.value !== "")
        {
          if (!childContent.classList.contains("show"))
          {
            content.classList.add("fade-out");
            setTimeout(() => {
              childContent.classList.add("show");
            }, 300);
            setTimeout(() => {
              content.classList.remove("fade-in");
              content.classList.remove("fade-out");
            }, 900);
          }
          searching = true;
          if (currentPage === "matBillSearch") search(searchTbx.value, "materials", true);
          else if (currentPage === "actBillSearch") search(searchTbx.value, "accounts", true);
          else if (currentPage === "actLastMoveSearch") search(searchTbx.value, "accounts", "lastMove");
          else search(searchTbx.value);
        }
        else if (currentPage !== "matBillSearch" && currentPage !== "actBillSearch" && currentPage !== "actLastMoveSearch")
        {
          that.table.body.id = DB.prototype.store("s:g", "parentPage");
          DB.prototype.store("s:s", "currentPage", DB.prototype.store("s:g", "parentPage"));
          if (childContent.classList.contains("show"))
          {
            childContent.classList.add("hide");
            setTimeout(() => {
              content.classList.add("fade-in");
            }, 300);
            setTimeout(() => {
              childContent.classList.remove("show");
              childContent.classList.remove("hide");
            }, 900);
          }
          searching = false;
        }
        else
        {
          searching = false;
          if (currentPage === "matBillSearch") search(searchTbx.value, "materials", true);
          else if (currentPage === "actBillSearch") search(searchTbx.value, "accounts", true);
          else if (currentPage === "actLastMoveSearch") search(searchTbx.value, "accounts", "lastMove");
        }
      }, 200);
    });

    
    // Back button click type event
    backBtn.addEventListener("click", that.backEvents);

    // Hide search bar
    wrapper.addEventListener("click", e => {
      if (e.path.indexOf(header) === -1)
      {
        if (searchBar.classList.contains("show-bar")) searchBar.classList.add("hide");
        setTimeout(() => {
          searchBar.classList.remove("hide");
          searchBar.classList.remove("show-bar");
          menuBtn.classList.remove("hide");
          searchBtn.classList.remove("hide");
          backBtn.classList.remove("hide");
          searchTbx.value = '';
        }, 490);
      }
    });

    //===============HIDDEN MENU SETTINGS===============//

    contentItem.forEach(item => {
      item.addEventListener("click", () => {
        contentItem.forEach(i => {
          i.classList.remove("active");
          if (childContent.classList.contains("show"))
          {
            childContent.classList.add("hide");
            setTimeout(() => {
              content.classList.add("fade-in");
            }, 300);
            setTimeout(() => {
              childContent.classList.remove("show");
              childContent.classList.remove("hide");
            }, 900);
          }
        });
        item.classList.add("active");
      });
    });

    // Settings option
    settingsBtn.addEventListener("click", () => {
      setTimeout(() => {
        settings.classList.add("show-settings");
      }, 290);
    });

    // Log out option
    logoutBtn.addEventListener("click", () => {
      subDb.close();
      subDb.store("l:r", "user");
      subDb.store("s:r", "user");
      subDb.store("l:r", "lastSession");
      subDb.store("l:r", "loggedin");
      window.location.reload();
    });

    // Home options 
    homeOpt.addEventListener("click", () => that.home());

    // Materials option
    materials.addEventListener("click", () => Tables.materials());

    // Accounts option
    accounts.addEventListener("click", () => Tables.accounts());
  }

  /** 
   * @private
   * Settings DOM functions 
   */  
  function SettingsFunctions (show = false)
  {
    const lhost   = subDb.store("l:g", "ip");
    const lport   = subDb.store("l:g", "port");
    const ldb     = subDb.store("l:g", "db");
    const host    = lhost !== null ? lhost : subDb.store("s:g", "ip");
    const port    = lport !== null ? lport : subDb.store("l:g", "port");
    const dbn     = ldb   !== null ? ldb   : subDb.store("l:g", "db");
    const hostTbx = document.getElementById("host");
    const portTbx = document.getElementById("port");
    const dbTbx   = document.getElementById("dbname");
    
    const settings      = document.getElementById("settings");
    const localLanguage = DB.prototype.store("l:g", "localLanguage");
    const langsList     = document.getElementById("langs-list");
    const langForm      = document.getElementById("lang-form");
    const currentLang   = document.querySelector("#set-lang.current-lang");
    const langItems     = document.querySelectorAll("#langs-list li");
    
    langForm.addEventListener("click", () => langsList.classList.remove("hide"));
    currentLang.innerHTML = spinal.language.codes[localLanguage];

    settings.addEventListener("click", e => {
      if (e.path.indexOf(document.querySelector("#set-lang.lang-list")) == -1 && e.path.indexOf(langsList) == -1) {
        if (langsList.classList.contains("hide") === false) {
          langsList.classList.add("hide");
        }
      }
    });

    langItems.forEach(item => {
      let code = item.getAttribute("code");
      item.innerHTML = spinal.language.codes[code];
      item.addEventListener("click", () => {
        DB.prototype.store("l:s", "localLanguage", code);
        currentLang.innerHTML = spinal.language.codes[code];
        window.location.reload(true);
      })
    })

    // Display connection info in Connection Settings fields
    hostTbx.value = host;
    portTbx.value = port;
    dbTbx.value   = dbn;

    // Load the changes in connection info as it change in Connection Settings fields
    hostTbx.addEventListener("input", () => subDb.store("l:s", "ip", hostTbx.value));
    portTbx.addEventListener("input", () => subDb.store("l:s", "port", portTbx.value));
    dbTbx.addEventListener("input", () => subDb.store("l:s", "db", dbTbx.value));

    let arrow = document.querySelector("#set-lang.lang-list span");
    let side = "right";
    if (localLang === "ar")
    {
      side = "left";
      settings.classList.add("right");
    }

    arrow.innerHTML = `<i class="fas fa-chevron-${side}"></i>`;
    
    // Make the back button in settings page force app to reload
    // if there is an error in connection info
    if(show === true)
    {
      document.getElementById("back").addEventListener("click", () => window.location.reload());
      return false;
    }

    let backBtn = document.getElementById("back");
    let setting = document.getElementById("settings");

    backBtn.addEventListener("click", () => {
      if (setting.classList.contains("show-settings")) setting.classList.add("hide");
      setTimeout(() => {
        setting.classList.remove("hide");
        setting.classList.remove("show-settings");
      }, 590);
    });
  }

  /** 
   * @private
   * The functions of home DOM template 
   */  
  function homeFunctions ()
  {
    document.getElementById("content").classList.add("fade-in");
    let fadeCard  = document.getElementById("fade-card");
    let username  = document.querySelector("#info h2");
    let company   = document.querySelector("#info #company");
    let dataName   = document.querySelector("#info #data-name");
    let matCount  = document.getElementById("mat-count");
    let actCount  = document.getElementById("act-count");
    let activeBtn = document.querySelector(".content-item.active");

    username.innerHTML = DB.prototype.store("l:g", "user");
    company.innerHTML  = DB.prototype.store("l:g", "company");
    dataName.innerHTML  = DB.prototype.store("l:g", "dataName");

    let matBtn = document.getElementById("mat-stick");
    let actBtn = document.getElementById("act-stick");
    let matBillBtn = document.getElementById("mat-button");
    let actBillBtn = document.getElementById("act-button");
    let actLastBtn = document.getElementById("act-last-button");

    if (localLang === "ar") fadeCard.classList.add("right");

    matCount.innerHTML = DB.prototype.store("s:g", "materialsLength");
    actCount.innerHTML = DB.prototype.store("s:g", "accountsLength");

    matBtn.addEventListener("click", () => {
      activeBtn.classList.remove("active");
      document.querySelector("#materials.content-item").classList.add("active");
      Tables.materials();
    });
    
    actBtn.addEventListener("click", () => {
      activeBtn.classList.remove("active");
      document.querySelector("#accounts.content-item").classList.add("active");
      Tables.accounts();
    });

    let content = document.getElementById("content");
    let billFuncs = [[matBillBtn, actBillBtn, actLastBtn],
                     ["materials", "accounts", "accounts"],
                     [true, true, "lastMove"]];
    
    for (let i = 0; i < billFuncs[0].length; i++) {
      billFuncs[0][i].addEventListener("click", () => {
      content.classList.add("fade-out");
      search('', billFuncs[1][i], billFuncs[2][i]);
      setTimeout(() => {
        content.classList.remove("fade-in");
        content.classList.remove("fade-out");
      }, 900);
      setTimeout(() => {
        document.getElementById("child-content").classList.add("show");
      }, 300);
    })
    }
    matBillBtn.addEventListener("click", () => {
      content.classList.add("fade-out");
      search('', "materials", true);
      setTimeout(() => {
        content.classList.remove("fade-in");
        content.classList.remove("fade-out");
      }, 900);
      setTimeout(() => {
        document.getElementById("child-content").classList.add("show");
      }, 300);
    });

    if (spinal.translated === true) spinal.translate();
  }

  /** 
   * @private
   * Display results that match with the search value
   * @param value   The search value 
   */  
  function search (value, key = DB.prototype.store("s:g", "parentPage"), bill = false)
  {
    let reg, result = [];
    for (let i = 0; i < db.data[key].length; i++)
    {
      if (db.data[key][i] !== -1) {
        reg = new RegExp(value.toString(), 'g');

        if (reg.test((db.data[key][i].ElmCode).toString()) || reg.test((db.data[key][i].ElmName).toString()))
          result.push(db.data[key][i]);
      }
    }
    
    let childContent = document.getElementById("child-content");
    if (key == "materials")
    {
      childContent.innerHTML = '';
      bill == false ? Tables.matSearch(result) : Tables.matBillSearch(result);
    }
    else if (key == "accounts")
    {
      childContent.innerHTML = '';
      if (bill == false) Tables.actSearch(result);
      else if (bill == false) Tables.actBillSearch(result);
      else Tables.actLastMoveSearch(result);
    }
  }

  DOM.prototype.addSearch = () => {
    let childContent = document.getElementById("child-content");
    childContent.style.height = "100%";
  }

  DOM.prototype.addFilters = parent => {
    document.querySelector("#child-content .content-body").style.height = "calc(100% - 208px)";

    dom.loadDOM("././templates/filters.html", html => {
      let bills        = document.getElementById(parent);
      let filter       = document.createElement("div");
      let contentChild = document.getElementById("child-content");

      filter.innerHTML = html;
      filter.classList.add("filter");
      contentChild.insertBefore(filter, bills);
  
      let filterShadow = document.createElement("div");
      filterShadow.id = "filter-shadow";
      contentChild.insertBefore(filterShadow, filter);
      filterShadow.addEventListener("click", () => {
        filter.classList.add("hide");
        setTimeout(() => {
          filter.classList.remove("show");
          filter.classList.remove("hide");
          filterShadow.classList.remove("show");
        }, 250);
      });

      let showFilter = document.createElement("div");
      showFilter.classList.add("show-filter");
      
      let showBtn = document.createElement("button");
      showBtn.innerText = spinal.language[localLang].filterResult;
      showFilter.appendChild(showBtn);
      showBtn.addEventListener("click", () => {
        filter.classList.add("show");
        filterShadow.classList.add("show");
      });

      let filterTitles = document.querySelectorAll(".filter-title");
      filterTitles[0].innerHTML = spinal.language[localLang].fromDate;
      filterTitles[1].innerHTML = spinal.language[localLang].toDate;

      contentChild.insertBefore(showFilter, bills);
  
      let elements  = document.querySelectorAll("#child-content .element .popup-card tr:first-child td:first-child");
      let filterBtn = document.getElementById("filter");
      let resetBtn  = document.getElementById("reset");

      filterBtn.innerHTML = spinal.language[localLang].filterBtn;
      resetBtn.innerHTML  = spinal.language[localLang].resetBtn;
  
      filterBtn.addEventListener("click", e => {
        let fromYear  = document.querySelector("select.filterItem.yyyy.from").value;
        let fromMonth = document.querySelector("select.filterItem.mm.from").value;
        let fromDay   = document.querySelector("select.filterItem.dd.from").value;
        let toYear    = document.querySelector("select.filterItem.yyyy.to").value;
        let toMonth   = document.querySelector("select.filterItem.mm.to").value;
        let toDay     = document.querySelector("select.filterItem.dd.to").value;
  
        elements.forEach(element => {
          let date = element.innerText.split('-');
          if (((Number(date[0]) >= Number(fromYear)  || isNaN(fromYear))  &&
               (Number(date[1]) >= Number(fromMonth) || isNaN(fromMonth)) &&
               (Number(date[2]) >= Number(fromDay)   || isNaN(fromDay)))  &&
              ((Number(date[0]) <= Number(toYear)    || isNaN(toYear))  &&
               (Number(date[1]) <= Number(toMonth)   || isNaN(toMonth)) &&
               (Number(date[2]) <= Number(toDay)     || isNaN(toDay)))
             )
          {
            element.closest(".element").style.display = "block";
          } else element.closest(".element").style.display = "none";
        });
        filter.classList.add("hide");
        setTimeout(() => {
          filter.classList.remove("show");
          filter.classList.remove("hide");
          filterShadow.classList.remove("show");
        }, 250);
      });
  
      resetBtn.addEventListener("click", () => {
        document.querySelectorAll("select.filterItem").forEach(element => {
          element.selectedIndex = 0;
        });
        filterBtn.click();
      });
    });
  }

  DOM.prototype.backEvents = () => {
    let currentPage = DB.prototype.store("s:g", "currentPage");
    let parentPage  = DB.prototype.store("s:g", "parentPage");
    let isSearching = DB.prototype.store("s:g", "searching");

    if (currentPage === "materials" || currentPage === "accounts")
    {
      document.querySelector(".content-item.active").classList.remove("active");
      document.querySelector("#homeOpt.content-item").classList.add("active");
      that.home();
    }
    else
    {
      if (parentPage !== null && parentPage !== currentPage && isSearching !== "true")
      {
        let content      = document.getElementById("content");
        let childContent = document.getElementById("child-content");

        DB.prototype.store("s:s", "currentPage", parentPage);
        childContent.classList.add("hide");
        setTimeout(() => {
          childContent.classList.remove("show");
          childContent.classList.remove("hide");
          while (childContent.firstChild) {
            childContent.removeChild(childContent.firstChild);
          }
          that.table.body.id = parentPage;
        }, 900);
        setTimeout(() => {
          content.classList.add("fade-in");
        }, 300);
      }
      else if (parentPage !== null && currentPage.substring(0, 3) === "mat" && isSearching !== "true") Tables.materials();
      else if (parentPage !== null && currentPage.substring(0, 3) === "act" && isSearching !== "true") Tables.accounts();
      else if (isSearching === "true") {
        let childContent = document.getElementById("child-content");

        DB.prototype.store("s:s", "currentPage", "home");
        childContent.classList.add("hide");
        setTimeout(() => {
          childContent.classList.remove("show");
          childContent.classList.remove("hide");
          while (childContent.firstChild) {
            childContent.removeChild(childContent.firstChild);
          }
          that.table.body.id = parentPage;
        }, 900);
        setTimeout(() => {
          that.home();
        }, 300);
      }
    }
  }

  return DOM;
})();