const Log = (() => {

  /** 
   * @private
   * @enum Allowed data types 
   */  
  const types = {
    s: "string",
    a: "array",
    o: "object",
    n: "number",
    i: "bigint",
    b: "boolean",
    f: "function",
    m: "symbol",
    u: "undefined",
    t: "iterator"
  }

  /** 
   * @private
   * Instance of the TableDOM class
   * @var Object 
   */ 
  var that;

  /** 
   * @public
   * @constructor
   */
  function Log ()
  {
    that = this;
  }

  /** 
   * @public
   * Get variable type
   * @param mix args  Variable to check type
   * @returns Variable type 
   */  
  Log.prototype.typeOf = (arg) =>
  {
    if ([arg][Symbol.iterator]() === arg) return "iterator";
    else if (Array.isArray(arg)) return "array";
    else return typeof arg;
  }

  /** 
   * @public
   * Throw typeError if the selected argument type is not the same with the given type
   * @param Object obj  The required type & the variable to check 
   * @throws typeError log
   * @example
   * let myVar = [];
   * new Log().checkType({"o:myVar":myVar}); //=> False
   */  
  Log.prototype.checkType = (obj, callback = null) => {
    try
    {
      /** 
       * Type of the local argument
       * @constant string 
       */
      const objType = that.typeOf(obj);

      // Check if the given argument is an object or not
      if (objType !== "object")
        throw "Invalid argument \"" + obj + "\"! \"obj\" must be an object. " + objType + " given.";

      for (let [type, arg] of Object.entries(obj))
      {
        // Check if the type string syntax is correct or not
        if(!type.includes(':'))
          throw "The type string must contain ':' between type code and argument name!";

        let typeCodes     = type.split(':');
        let argType       = that.typeOf(arg);
        let requiredType  = types[typeCodes[0]];

        // Check if the current argument matches the required type
        if (argType !== requiredType)
          throw "Invalid argument \"" + arg + "\"! The argument " + typeCodes[1] +
          " must be a/an \"" + requiredType + "\". \"" + argType + "\" given.";
      }

      if (callback !== null) callback(true);
    }
    catch (error)
    {
      console.error(error);
      if (callback !== null) callback(false);
      else return false;
    }
  }

  return  Log;
})();