const Spinal = (() => {

  /** 
   * @private
   * The specific title of table DOM class errors
   * @constant string 
   */ 
  const errorTitle = "";

  /** 
   * @private
   * New instance of @class Log
   * @constant Object
   */ 
  const log = new Log();

  /** 
   * @private
   * Instance of the TableDOM class
   * @var Object 
   */  
  var that;
  
  /** 
   * @public
   * The language data for the application
   * @var Object 
   */  
  Spinal.prototype.language;

  /** 
   * @public
   * @constructor 
   */  
  function Spinal (settings = null)
  {
    try
    {
      if (settings !== null)
      {
        if (checkSettings({
          "s:languageFile": settings.languageFile
        }) !== true) throw new Error("There was a syntax error in the settings object. Or some settings were missing.");
        
        that = this;
        loadLanguage(settings.languageFile);
      }

      /** 
       * @public
       * has the app been translate or not?
       * @var boolean 
       */  
      this.translated = false;
    }
    catch (error)
    {
      console.error(error);
      return false;
    }
  }

  /** 
   * @public
   * Execute multiple methods in order
   * @param iterable  Methods-containing array
   * @param callback  The callback to get the iterable methods as arguments (optional)
   * @returns void|Mix
   */  
  Spinal.prototype.iterateOver = (iterable, callback = null) => {
    if (log.checkType({"a:iterable":iterable}) === false) return;

    try
    {
      if (callback !== null)
      {
        if (log.checkType({"f:callback":callback}) === false) return;

        iterable.forEach(m => callback(m));
      }
      else
      {
        for (let i = 0; i < iterable.length; i++) queue.push(iterable[i]);
        that.execute();
      }
    }
    catch (error)
    {
      console.error(errorTitle + error);
      return false;
    }
  }

  /** 
   * @private
   * Load language data
   * @requires file.json
   * @param path  The path for the JSON mode file
   */  
  function loadLanguage (path)
  {
    if (log.checkType({"s:path":path}) === false) return;
    
    try
    {
      if (!path.includes(".json"))
        throw new Error("Invalid path \"" + path + "\"! Requires a JSON mode file with \".json\" extention.");
      $.getJSON(path, data => that.language = data);
    }
    catch (error)
    {
      console.error(error);
      return false;
    }
  }

  /** 
   * @private
   * Check for data type of the settings object values
   * @param settings  Object with the values and required data types to check
   * @returns boolean
   */  
  function checkSettings (settings)
  {
    if (log.checkType({"o:settings":settings}) === false) return;

    try
    {
      for (const key in settings)
      {
        if (settings.hasOwnProperty(key))
        {
          const value = settings[key];
          if (typeof value === "undefined") throw new Error("Requires " + key.substring(2) + "!");

          if (log.checkType({[key]:value}) === false) return;
        }
      }
      return true;
    }
    catch (error)
    {
      console.error(error);
      return false;
    }
  }
  
  /** 
   * @public
   * Callback the index 0 of the @global @vaer "queue"
   * @returns queue void 
   */  
  Spinal.prototype.execute = () => {
    try
    {
      if (log.typeOf(queue) === "array" && queue.length > 0)
      {
        let exc = queue[0];

        // Remove the first value of queue
        queue.shift();

        if (log.typeOf(exc) === "object")
        {
          for (let [func, args] of Object.entries(exc))
          {
            eval(func)(...args);
          }
        }
        else exc();
      }
    }
    catch (error)
    {
      console.error(error);
      return false;
    }
  }

  /** 
   * @public
   * Translate strings in the application
   * @returns setLanguage() 
   */  
  Spinal.prototype.translate = () => {

    let localLanguage = localStorage.getItem("localLanguage");
    let browserLang = navigator.language;
    if(!localLanguage) {
      if (that.language.hasOwnProperty(browserLang)) {
        setLanguage (browserLang);
      } else setLanguage ("en");
    } else setLanguage (localLanguage);
    that.translated = true;
  }

  /** 
   * @private
   * Get the language code from @function translate and
   * and rewrite strings with that langauge
   * @param code  The code of the language
   */  
  function setLanguage (code) {
    localStorage.setItem("localLanguage", code);
    langs = document.querySelectorAll(".lang");
    langs.forEach(lang => {
      lang.innerText = that.language[code][lang.getAttribute("key")];
    });
  };

  return Spinal;
})();