
//=================Import classes=================//

/**
 * @global
 * The queue of ordered methods to iterate
 * @var array
 */
let queue = [];

/** 
 * @global
 * New instance of @class DB
 * @constant object 
 */
const db = new DB();

/** 
 * @global
 * New instance of @class Spinal
 * @constant object 
 */
const spinal = new Spinal({
  languageFile: "js/json/language.json"
})

/** 
 * @global
 * New instance of @class DOM
 * @constant object 
 */
const dom = new DOM("app");

/** 
 * @global
 * The partical of the ordered methods to iterate over
 * 
 * @var array 
 */
var partical = [
  db.open,
  db.login,
  db.cacheMaterials,
  db.cacheAccounts,
  db.warehouses,
  initialize
];
// Start execution when document is loaded
document.addEventListener('DOMContentLoaded', () =>  spinal.iterateOver(partical), false);

/** 
 * @global
 * Initialize the actual functionality of the application right after
 * the cordova libraries is loaded 
 * 
 * @returns void
 */
function initialize()
{
  document.addEventListener('deviceready', onDeviceReady, false);
}

/** 
 * @global
 * Execute function when device is ready
 * @returns iterator 
 */
function onDeviceReady()
{
  var pages = [
    dom.settings,
    dom.body,
    dom.home,
    dom.functions,
    spinal.translate
  ];

  if (db.store("l:g", "loggedin") == "true") spinal.iterateOver(pages);
}

//=================DATA TABLES=================//

const colors = {
  materials: "#1c3a54",
  accounts: "#2D6673",
  matBills: "#9c2121",
  matDescriptors: "#323C40",
  actBills: "#9c2121",
  actDescriptors: "#323C40",
  matSearch: "#1c3a54",
  matBillSearch: "#1c3a54",
  actSearch: "#2D6673",
  actBillSearch: "#2D6673",
  actLastMoveSearch: "#2D6673",
  "matwarehouses": "#4ac34e"
}

let localLang = db.store("l:g", "localLanguage");


const Tables = {

  materials: () => {
    document.addEventListener("backbutton", dom.backEvents, false);
    db.store("s:s", "NameSelected", "Materials");
    db.store("s:s", "parentPage", "materials");
    dom.showBack();
    dom.table.create("#content", "materials");
    dom.table.scrollLoader(db.data.materials, [
      ["<i class=\"fas fa-exchange-alt\"></i>", "bills", Tables.matBills],
      ["<i class=\"fas fa-info-circle\"></i>", "descriptors", Tables.matDescriptors],
      ["<i class=\"fas fa-warehouse\"></i>", "warehouses", Tables.matwarehouses]
    ],
    {name: "ElmCode", key: "code"},
    {name: "ElmName", key: "name"}, false,
    {name: spinal.language[localLang].balance, sub:["MttDebit", "MttCredit"]});
  },

  accounts: () => {
    document.addEventListener("backbutton", dom.backEvents, false);
    db.store("s:s", "NameSelected", "Accounts");
    db.store("s:s", "parentPage", "accounts");
    dom.showBack();
    dom.table.create("#content", "accounts");
    dom.table.scrollLoader(db.data.accounts, [
      ["<i class=\"fas fa-money-check-alt\"></i>", "actBills", Tables.actBills],
      ["<i class=\"fas fa-info-circle\"></i>", "descriptors", Tables.actDescriptors],
      ["<i class=\"fas fa-history\"></i>", "history", Tables.actLastMoves]
    ],
    {name: "ElmCode", key: "code"},
    {name: "ElmName", key: "name"}, false,
    {name: spinal.language[localLang].balance, sub:["ActDebit", "ActCredit"]});
  },

  matwarehouses: () => {
    dom.showBack();
    db.store("s:s", "parentPage", "materials");
    db.query("matwarehouses", ["mat=" + db.store("s:g", "ElmSelected")], data => {
      for (let i = 0; i < data.length; i++) {
        data[i].BtmWarehouse = db.data.warehouses.get(data[i].BtmWarehouse);
        data[i]["balance"] = Number(data[i].BtmDebit) - Number(data[i].BtmCredit);
      }
      dom.table.create("#child-content", "matwarehouses");
      dom.table.fill(data, false,
        {name: "balance", key: "balance"},
        {name: "BtmWarehouse", key: "warehouseName"}, false,
        {name: spinal.language[localLang].matCredit, key: "BtmCredit", color:"red"},
        {name: spinal.language[localLang].matDebit, key: "BtmDebit", color:"#00ce00"});
    });
  },

  matBills: () => {
    dom.showBack();
    db.store("s:s", "parentPage", "materials");
    db.query("repbilitems", ["mat=" + db.store("s:g", "ElmSelected")], data => {
      for (let i = 0; i < data.length; i++) {
        data[i].BtmWarehouse = db.data.warehouses.get(data[i].BtmWarehouse);
        data[i].BtmAccount   = db.data.accounts[data[i].BtmAccount].ElmName;
      }
      dom.table.create("#child-content", "matBills", true);
      dom.table.fill(data, false,
        {name: "BtmDate", key: "date"}, "MunName", false,
        {name: spinal.language[localLang].balance, sub:["BtmDebit", "BtmCredit"], display: false},
        {name: spinal.language[localLang].matCredit, key: "BtmCredit", color:"red", hide: 0},
        {name: spinal.language[localLang].matDebit, key: "BtmDebit", color:"#00ce00", hide: 0},
        {name: spinal.language[localLang].total, key: "BtmTotal"},
        {name: spinal.language[localLang].warehouse, key: "BtmWarehouse"},
        {name: spinal.language[localLang].account, key: "BtmAccount"});
        dom.addFilters("matBills");
    });
  },
  
  matDescriptors: () => {
    dom.showBack();
    db.store("s:s", "parentPage", "materials");
    db.query("matdescriptors", ["mat=" + db.store("s:g", "ElmSelected")], data => {
      dom.table.create("#child-content", "matDescriptors");
      dom.table.fill(data, false, "DscdDscName", null, false, "DscdText");
    });
  },

  actBills: () => {
    dom.showBack();
    db.query("repvouitems", ["acc=" + db.store("s:g", "ElmSelected")], data => {
      dom.table.create("#child-content", "actBills", true);
      dom.table.fill(data, false,
        {name: "ItmDate", key: "date"},
        "MunName", false,
        {name: spinal.language[localLang].balance, sub:["ItmDebit", "ItmCredit"], display: false},
        {name: spinal.language[localLang].actCredit, key: "ItmCredit", color:"red", hide: 0},
        {name: spinal.language[localLang].actDebit, key: "ItmDebit", color:"#00ce00", hide: 0});
      dom.addFilters("actBills")
    });
  },

  actLastMoves: () => {
    dom.showBack();
    db.query("repvouitems", ["acc=" + db.store("s:g", "ElmSelected")], data => {
      dom.table.create("#child-content", "actBills", true);
      dom.table.fill(data, false,
        {name: "ItmDate", key: "date"},
        "MunName", true,
        {name: spinal.language[localLang].balance, sub:["ItmDebit", "ItmCredit"], display: false},
        {name: spinal.language[localLang].actCredit, key: "ItmCredit", color:"red", hide: 0},
        {name: spinal.language[localLang].actDebit, key: "ItmDebit", color:"#00ce00", hide: 0});
      dom.addFilters("actBills")
    });
  },

  actDescriptors: () => {
    dom.showBack();
    db.query("accdescriptors", ["acc=" + db.store("s:g", "ElmSelected")], data => {
      dom.table.create("#child-content", "actDescriptors");
      dom.table.fill(data, false, "DscdDscName", null, false, "DscdText");
    });
  },

  matSearch: (data) => {
    dom.table.create("#child-content", "matSearch");
    dom.table.scrollLoader(data, [
      ["<i class=\"fas fa-exchange-alt\"></i>", "bills", Tables.matBills],
      ["<i class=\"fas fa-info-circle\"></i>", "descriptors", Tables.matDescriptors],
      ["<i class=\"fas fa-warehouse\"></i>", "warehouses", Tables.matwarehouses]
    ],
    {name: "ElmCode", key: "code"},
    {name: "ElmName", key: "name"}, false,
    {name: spinal.language[localLang].balance, sub:["MttDebit", "MttCredit"]});
  },

  actSearch: (data) => {
    dom.table.create("#child-content", "actSearch");
    dom.table.scrollLoader(data, [
      ["<i class=\"fas fa-money-check-alt\"></i>", "bills", Tables.actBills],
      ["<i class=\"fas fa-info-circle\"></i>", "descriptors", Tables.actDescriptors],
      ["<i class=\"fas fa-history\"></i>", "history", Tables.actLastMoves]
    ],
    {name: "ElmCode", key: "code"},
    {name: "ElmName", key: "name"}, false,
    {name: spinal.language[localLang].balance, sub:["ActDebit", "ActCredit"]});
  },

  matBillSearch: (data) => {
    document.addEventListener("backbutton", dom.backEvents, false);
    db.store("s:s", "searching", "true");
    dom.showBack();
    dom.table.create("#child-content", "matBillSearch");
    dom.table.scrollLoader(data, false,
    {name: "ElmCode", key: "code"},
    {name: "ElmName", key: "name"}, false,
    {name: spinal.language[localLang].balance, sub:["MttDebit", "MttCredit"]});
    dom.addSearch();
  },

  actBillSearch: (data) => {
    document.addEventListener("backbutton", dom.backEvents, false);
    db.store("s:s", "searching", "true");
    dom.showBack();
    dom.table.create("#child-content", "actBillSearch");
    dom.table.scrollLoader(data, false,
    {name: "ElmCode", key: "code"},
    {name: "ElmName", key: "name"}, false,
    {name: spinal.language[localLang].balance, sub:["ActDebit", "ActCredit"]});
    dom.addSearch();
  },

  actLastMoveSearch: (data) => {
    document.addEventListener("backbutton", dom.backEvents, false);
    db.store("s:s", "searching", "true");
    dom.showBack();
    dom.table.create("#child-content", "actLastMoveSearch");
    dom.table.scrollLoader(data, false,
    {name: "ElmCode", key: "code"},
    {name: "ElmName", key: "name"}, false,
    {name: spinal.language[localLang].balance, sub:["ActDebit", "ActCredit"]});
    dom.addSearch();
  }
};